@@ -48,7 +48,7 @@ class IntervalPartition : public FunctionPass {
 public:
   static char ID; // Pass identification, replacement for typeid
 
-  IntervalPartition() : FunctionPass((intptr_t)&ID), RootInterval(0) {}
+  IntervalPartition() : FunctionPass(&ID), RootInterval(0) {}
 
   // run - Calculate the interval partition for this function
   virtual bool runOnFunction(Function &F);
@@ -59,14 +59,8 @@ public:
   //
   IntervalPartition(IntervalPartition &I, bool);
 
-  // Destructor - Free memory
-  ~IntervalPartition() { destroy(); }
-
   // print - Show contents in human readable format...
-  virtual void print(std::ostream &O, const Module* = 0) const;
-  void print(std::ostream *O, const Module* M = 0) const {
-    if (O) print(*O, M);
-  }
+  virtual void print(raw_ostream &O, const Module* = 0) const;
 
   // getRootInterval() - Return the root interval that contains the starting
   // block of the function.
@@ -92,10 +86,10 @@ public:
   // Interface to Intervals vector...
   const std::vector<Interval*> &getIntervals() const { return Intervals; }
 
-private:
-  // destroy - Reset state back to before function was analyzed
-  void destroy();
+  // releaseMemory - Reset state back to before function was analyzed
+  void releaseMemory();
 
+private:
   // addIntervalToPartition - Add an interval to the internal list of intervals,
   // and then add mappings from all of the basic blocks in the interval to the
   // interval itself (in the IntervalMap).
