@@ -145,12 +145,11 @@ static void DumpSymbolNamesFromFile(std::string &Filename) {
     if (Buffer.get())
       Result = ParseBitcodeFile(Buffer.get(), Context, &ErrorMessage);
     
-    if (Result)
+    if (Result) {
       DumpSymbolNamesFromModule(Result);
-    else {
+      delete Result;
+    } else
       errs() << ToolName << ": " << Filename << ": " << ErrorMessage << "\n";
-      return;
-    }
     
   } else if (aPath.isArchive()) {
     std::string ErrMsg;
