@@ -13,22 +13,39 @@
 //
 //===----------------------------------------------------------------------===//
 
+#include "llvm/Config/config.h"
 #include "llvm/CompilerDriver/BuiltinOptions.h"
 #include "llvm/CompilerDriver/ForceLinkage.h"
 #include "llvm/System/Path.h"
+#include <iostream>
 
 namespace llvmc {
   int Main(int argc, char** argv);
 }
 
+// Modify the PACKAGE_VERSION to use build number in top level configure file.
+void PIC16VersionPrinter(void) {
+  std::cout << "MPLAB C16 1.0 " << PACKAGE_VERSION << "\n";
+}
+
 int main(int argc, char** argv) {
 
   // HACK
   SaveTemps.setHiddenFlag(llvm::cl::Hidden);
-  TempDirname = "tmp-objs";
+  TempDirname.setHiddenFlag(llvm::cl::Hidden);
+  Languages.setHiddenFlag(llvm::cl::Hidden);
+  DryRun.setHiddenFlag(llvm::cl::Hidden);
 
-  // Remove the temp dir if already exists.
+  llvm::cl::SetVersionPrinter(PIC16VersionPrinter); 
+
+  // Ask for a standard temp dir, but just cache its basename., and delete it.
   llvm::sys::Path tempDir;
+  tempDir = llvm::sys::Path::GetTemporaryDirectory();
+  TempDirname = tempDir.getBasename();
+  tempDir.eraseFromDisk(true);
+
+  // We are creating a temp dir in current dir, with the cached name.
+  //  But before that remove if one already exists with that name..
   tempDir = TempDirname;
   tempDir.eraseFromDisk(true);
 
